import numpy as np
import cv2


# params for ShiTomasi corner detection
feature_params = dict(qualityLevel = 0.3,
                       minDistance = 7,
                       blockSize = 7 )

# Parameters for lucas kanade optical flow
lk_params = dict( winSize  = (15,15),
                  maxLevel = 2,
                  criteria = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 100, 0.03))

color = np.random.randint(0,255,(100,3))

cap = cv2.VideoCapture(0)
ret, frame = cap.read()
prev = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
p0 = cv2.goodFeaturesToTrack(prev, 300, mask = None, **feature_params)
mask = np.zeros_like(frame)

while(True):
    ret, frame = cap.read()
    next = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    frame = cv2.cvtColor(next, cv2.COLOR_GRAY2BGR)
        
    p1,st,err = cv2.calcOpticalFlowPyrLK(prev, next, p0, None, **lk_params)
    
    # Select good points
    good_new = p1[st==1]
    good_old = p0[st==1]

    # draw the tracks
    for i,(new,old) in enumerate(zip(good_new,good_old)):
        a,b = new.ravel()
        c,d = old.ravel()
        cv2.line(frame, (a,b),(c,d),255, 2)
        cv2.circle(frame,(a,b),3, 255,-1)
        
    cv2.imshow('frame',frame)
    if cv2.waitKey(1) > 0:
        break
    prev = next;
    p0 = cv2.goodFeaturesToTrack(prev, 300, mask = None, **feature_params)

    
cap.release()
cv2.destroyAllWindows()
